/****************************************************************************
** Meta object code from reading C++ file 'abstractapi.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../plasmatube-26.08.0/src/api/abstractapi.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'abstractapi.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10QInvidious11AbstractApiE_t {};
} // unnamed namespace

template <> constexpr inline auto QInvidious::AbstractApi::qt_create_metaobjectdata<qt_meta_tag_ZN10QInvidious11AbstractApiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QInvidious::AbstractApi",
        "credentialsChanged",
        "",
        "apiHostChanged",
        "canLogInChanged",
        "apiProtocolChanged",
        "SupportedFeature",
        "PopularPage",
        "TrendingCategories",
        "SponsorBlock",
        "RelatedVideos"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'credentialsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'apiHostChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canLogInChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'apiProtocolChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SupportedFeature'
        QtMocHelpers::EnumData<enum SupportedFeature>(6, 6, QMC::EnumFlags{}).add({
            {    7, SupportedFeature::PopularPage },
            {    8, SupportedFeature::TrendingCategories },
            {    9, SupportedFeature::SponsorBlock },
            {   10, SupportedFeature::RelatedVideos },
        }),
    };
    return QtMocHelpers::metaObjectData<AbstractApi, qt_meta_tag_ZN10QInvidious11AbstractApiE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QInvidious::AbstractApi::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QInvidious11AbstractApiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QInvidious11AbstractApiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10QInvidious11AbstractApiE_t>.metaTypes,
    nullptr
} };

void QInvidious::AbstractApi::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractApi *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->credentialsChanged(); break;
        case 1: _t->apiHostChanged(); break;
        case 2: _t->canLogInChanged(); break;
        case 3: _t->apiProtocolChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AbstractApi::*)()>(_a, &AbstractApi::credentialsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractApi::*)()>(_a, &AbstractApi::apiHostChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractApi::*)()>(_a, &AbstractApi::canLogInChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractApi::*)()>(_a, &AbstractApi::apiProtocolChanged, 3))
            return;
    }
}

const QMetaObject *QInvidious::AbstractApi::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QInvidious::AbstractApi::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10QInvidious11AbstractApiE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QInvidious::AbstractApi::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QInvidious::AbstractApi::credentialsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QInvidious::AbstractApi::apiHostChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QInvidious::AbstractApi::canLogInChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QInvidious::AbstractApi::apiProtocolChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
