/****************************************************************************
** Meta object code from reading C++ file 'videolistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasmatube-26.08.0/src/models/videolistmodel.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'videolistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14VideoListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto VideoListModel::qt_create_metaobjectdata<qt_meta_tag_ZN14VideoListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "VideoListModel",
        "QML.Element",
        "auto",
        "titleChanged",
        "",
        "requestChannel",
        "ucid",
        "requestQuery",
        "VideoListModel::QueryType",
        "type",
        "reset",
        "requestPlaylist",
        "id",
        "refresh",
        "markAsWatched",
        "index",
        "markAsUnwatched",
        "removeFromPlaylist",
        "plid",
        "queryTypeString",
        "QueryType",
        "queryTypeIcon",
        "title",
        "NoQuery",
        "Feed",
        "Search",
        "Trending",
        "TrendingGaming",
        "TrendingMovies",
        "TrendingMusic",
        "TrendingNews",
        "RecommendedVideos",
        "Channel",
        "History",
        "Playlist"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestChannel'
        QtMocHelpers::SlotData<void(const QString &)>(5, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'requestQuery'
        QtMocHelpers::SlotData<void(VideoListModel::QueryType, bool)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Bool, 10 },
        }}),
        // Slot 'requestPlaylist'
        QtMocHelpers::SlotData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'refresh'
        QtMocHelpers::SlotData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'markAsWatched'
        QtMocHelpers::SlotData<void(int)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'markAsUnwatched'
        QtMocHelpers::SlotData<void(int)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'removeFromPlaylist'
        QtMocHelpers::SlotData<void(const QString &, int)>(17, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::Int, 15 },
        }}),
        // Method 'queryTypeString'
        QtMocHelpers::MethodData<QString(enum QueryType)>(19, 4, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 20, 4 },
        }}),
        // Method 'queryTypeIcon'
        QtMocHelpers::MethodData<QString(enum QueryType)>(21, 4, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 20, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'title'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'QueryType'
        QtMocHelpers::EnumData<enum QueryType>(20, 20, QMC::EnumFlags{}).add({
            {   23, QueryType::NoQuery },
            {   24, QueryType::Feed },
            {   25, QueryType::Search },
            {   26, QueryType::Trending },
            {   27, QueryType::TrendingGaming },
            {   28, QueryType::TrendingMovies },
            {   29, QueryType::TrendingMusic },
            {   30, QueryType::TrendingNews },
            {   31, QueryType::RecommendedVideos },
            {   32, QueryType::Channel },
            {   33, QueryType::History },
            {   34, QueryType::Playlist },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<VideoListModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject VideoListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14VideoListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14VideoListModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14VideoListModelE_t>.metaTypes,
    nullptr
} };

void VideoListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VideoListModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->titleChanged(); break;
        case 1: _t->requestChannel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->requestQuery((*reinterpret_cast<std::add_pointer_t<VideoListModel::QueryType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->requestPlaylist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->refresh(); break;
        case 5: _t->markAsWatched((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->markAsUnwatched((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->removeFromPlaylist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: { QString _r = _t->queryTypeString((*reinterpret_cast<std::add_pointer_t<enum QueryType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->queryTypeIcon((*reinterpret_cast<std::add_pointer_t<enum QueryType>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VideoListModel::*)()>(_a, &VideoListModel::titleChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        default: break;
        }
    }
}

const QMetaObject *VideoListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VideoListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14VideoListModelE_t>.strings))
        return static_cast<void*>(this);
    return AbstractListModel::qt_metacast(_clname);
}

int VideoListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void VideoListModel::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
