// This file is autogenerated. Changes will be overwritten.
#include "EJRQKI7XPS/moc_config.cpp"
#include "EJRQKI7XPS/moc_sourceconfig.cpp"
#include "XNXWGTNIES/moc_mediaplayer2.cpp"
#include "XNXWGTNIES/moc_mediaplayer2player.cpp"
#include "XNXWGTNIES/moc_metadata.cpp"
#include "XNXWGTNIES/moc_mpris2.cpp"
