/****************************************************************************
** Meta object code from reading C++ file 'videomodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasmatube-26.08.0/src/models/videomodel.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'videomodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9VideoItemE_t {};
} // unnamed namespace

template <> constexpr inline auto VideoItem::qt_create_metaobjectdata<qt_meta_tag_ZN9VideoItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "VideoItem",
        "QML.Element",
        "anonymous",
        "thumbnailUrl",
        "QUrl",
        "",
        "quality",
        "authorThumbnail",
        "size",
        "recommendedVideosModel",
        "VideoListModel*",
        "isLoaded",
        "videoId",
        "title",
        "length",
        "viewCount",
        "author",
        "authorId",
        "authorUrl",
        "published",
        "publishedText",
        "description",
        "descriptionHtml",
        "liveNow",
        "paid",
        "premium",
        "keywords",
        "likeCount",
        "dislikeCount",
        "isFamilyFriendly",
        "allowedRegions",
        "genre",
        "genreUrl",
        "subCount",
        "allowRatings",
        "rating",
        "isListed",
        "url"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'thumbnailUrl'
        QtMocHelpers::MethodData<QUrl(const QString &) const>(3, 5, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'authorThumbnail'
        QtMocHelpers::MethodData<QUrl(quint32) const>(7, 5, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::UInt, 8 },
        }}),
        // Method 'recommendedVideosModel'
        QtMocHelpers::MethodData<VideoListModel *()>(9, 5, QMC::AccessPublic, 0x80000000 | 10),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isLoaded'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'videoId'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'length'
        QtMocHelpers::PropertyData<qint64>(14, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewCount'
        QtMocHelpers::PropertyData<qint64>(15, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'author'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'authorId'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'authorUrl'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'published'
        QtMocHelpers::PropertyData<QDateTime>(19, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'publishedText'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'descriptionHtml'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'liveNow'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'paid'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'premium'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'keywords'
        QtMocHelpers::PropertyData<QStringList>(26, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'likeCount'
        QtMocHelpers::PropertyData<qint32>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dislikeCount'
        QtMocHelpers::PropertyData<qint32>(28, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isFamilyFriendly'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'allowedRegions'
        QtMocHelpers::PropertyData<QStringList>(30, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'genre'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'genreUrl'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'subCount'
        QtMocHelpers::PropertyData<qint32>(33, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'allowRatings'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'rating'
        QtMocHelpers::PropertyData<double>(35, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isListed'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'url'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<VideoItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject VideoItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9VideoItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9VideoItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9VideoItemE_t>.metaTypes,
    nullptr
} };

void VideoItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VideoItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QUrl _r = _t->thumbnailUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 1: { QUrl _r = _t->authorThumbnail((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 2: { VideoListModel* _r = _t->recommendedVideosModel();
            if (_a[0]) *reinterpret_cast<VideoListModel**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isLoaded(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->videoId(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 3: *reinterpret_cast<qint64*>(_v) = _t->length(); break;
        case 4: *reinterpret_cast<qint64*>(_v) = _t->viewCount(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->author(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->authorId(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->authorUrl(); break;
        case 8: *reinterpret_cast<QDateTime*>(_v) = _t->published(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->publishedText(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->descriptionHtml(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->liveNow(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->paid(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->premium(); break;
        case 15: *reinterpret_cast<QStringList*>(_v) = _t->keywords(); break;
        case 16: *reinterpret_cast<qint32*>(_v) = _t->likeCount(); break;
        case 17: *reinterpret_cast<qint32*>(_v) = _t->dislikeCount(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isFamilyFriendly(); break;
        case 19: *reinterpret_cast<QStringList*>(_v) = _t->allowedRegions(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->genre(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->genreUrl(); break;
        case 22: *reinterpret_cast<qint32*>(_v) = _t->subCount(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->allowRatings(); break;
        case 24: *reinterpret_cast<double*>(_v) = _t->rating(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->isListed(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->url(); break;
        default: break;
        }
    }
}

const QMetaObject *VideoItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VideoItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9VideoItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QInvidious::Video"))
        return static_cast< QInvidious::Video*>(this);
    return QObject::qt_metacast(_clname);
}

int VideoItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN10VideoModelE_t {};
} // unnamed namespace

template <> constexpr inline auto VideoModel::qt_create_metaobjectdata<qt_meta_tag_ZN10VideoModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "VideoModel",
        "QML.Element",
        "auto",
        "isLoadingChanged",
        "",
        "videoIdChanged",
        "videoChanged",
        "audioUrlChanged",
        "remoteUrlChanged",
        "formatListChanged",
        "selectedFormatChanged",
        "errorOccurred",
        "errorText",
        "getVideoUrl",
        "isLoading",
        "videoId",
        "remoteUrl",
        "audioUrl",
        "video",
        "VideoItem*",
        "formatList",
        "selectedFormat"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'isLoadingChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoIdChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'audioUrlChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'remoteUrlChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'formatListChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedFormatChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Method 'getVideoUrl'
        QtMocHelpers::MethodData<QString() const>(13, 4, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isLoading'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'videoId'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'remoteUrl'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'audioUrl'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags, 4),
        // property 'video'
        QtMocHelpers::PropertyData<VideoItem*>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 2),
        // property 'formatList'
        QtMocHelpers::PropertyData<QStringList>(20, QMetaType::QStringList, QMC::DefaultPropertyFlags, 5),
        // property 'selectedFormat'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<VideoModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject VideoModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10VideoModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10VideoModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10VideoModelE_t>.metaTypes,
    nullptr
} };

void VideoModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VideoModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->isLoadingChanged(); break;
        case 1: _t->videoIdChanged(); break;
        case 2: _t->videoChanged(); break;
        case 3: _t->audioUrlChanged(); break;
        case 4: _t->remoteUrlChanged(); break;
        case 5: _t->formatListChanged(); break;
        case 6: _t->selectedFormatChanged(); break;
        case 7: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: { QString _r = _t->getVideoUrl();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)()>(_a, &VideoModel::isLoadingChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)()>(_a, &VideoModel::videoIdChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)()>(_a, &VideoModel::videoChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)()>(_a, &VideoModel::audioUrlChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)()>(_a, &VideoModel::remoteUrlChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)()>(_a, &VideoModel::formatListChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)()>(_a, &VideoModel::selectedFormatChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoModel::*)(const QString & )>(_a, &VideoModel::errorOccurred, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VideoItem* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isLoading(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->m_videoId; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->remoteUrl(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->audioUrl(); break;
        case 4: *reinterpret_cast<VideoItem**>(_v) = _t->m_video; break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->formatList(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->selectedFormat(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1:
            if (QtMocHelpers::setProperty(_t->m_videoId, *reinterpret_cast<QString*>(_v)))
                Q_EMIT _t->videoIdChanged();
            break;
        case 4:
            if (QtMocHelpers::setProperty(_t->m_video, *reinterpret_cast<VideoItem**>(_v)))
                Q_EMIT _t->videoChanged();
            break;
        case 6: _t->setSelectedFormat(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *VideoModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VideoModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10VideoModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int VideoModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void VideoModel::isLoadingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void VideoModel::videoIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void VideoModel::videoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void VideoModel::audioUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void VideoModel::remoteUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void VideoModel::formatListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void VideoModel::selectedFormatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void VideoModel::errorOccurred(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
