/****************************************************************************
** Meta object code from reading C++ file 'searchparameters.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../plasmatube-26.08.0/src/api/searchparameters.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'searchparameters.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16SearchParametersE_t {};
} // unnamed namespace

template <> constexpr inline auto SearchParameters::qt_create_metaobjectdata<qt_meta_tag_ZN16SearchParametersE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SearchParameters",
        "QML.Element",
        "auto",
        "query",
        "sortBy",
        "SortBy",
        "date",
        "Date",
        "duration",
        "Duration",
        "type",
        "Type",
        "features",
        "QList<Feature>",
        "pipedFilter",
        "PipedFilter",
        "Default",
        "Relevance",
        "Rating",
        "UploadDate",
        "ViewCount",
        "Hour",
        "Today",
        "Week",
        "Month",
        "Year",
        "Short",
        "Long",
        "Video",
        "Playlist",
        "Channel",
        "All",
        "Feature",
        "Hd",
        "Subtitles",
        "CreativeCommons",
        "ThreeD",
        "Live",
        "Purchased",
        "FourK",
        "ThreeSixty",
        "Location",
        "Hdr",
        "Videos",
        "Channels",
        "Playlists",
        "MusicSongs",
        "MusicVideos",
        "MusicAlbums",
        "MusicPlaylists",
        "MusicArtists"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'query'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'sortBy'
        QtMocHelpers::PropertyData<enum SortBy>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'date'
        QtMocHelpers::PropertyData<enum Date>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'duration'
        QtMocHelpers::PropertyData<enum Duration>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'type'
        QtMocHelpers::PropertyData<enum Type>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'features'
        QtMocHelpers::PropertyData<QList<Feature>>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'pipedFilter'
        QtMocHelpers::PropertyData<enum PipedFilter>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SortBy'
        QtMocHelpers::EnumData<enum SortBy>(5, 5, QMC::EnumIsScoped).add({
            {   16, SortBy::Default },
            {   17, SortBy::Relevance },
            {   18, SortBy::Rating },
            {   19, SortBy::UploadDate },
            {   20, SortBy::ViewCount },
        }),
        // enum 'Date'
        QtMocHelpers::EnumData<enum Date>(7, 7, QMC::EnumIsScoped).add({
            {   16, Date::Default },
            {   21, Date::Hour },
            {   22, Date::Today },
            {   23, Date::Week },
            {   24, Date::Month },
            {   25, Date::Year },
        }),
        // enum 'Duration'
        QtMocHelpers::EnumData<enum Duration>(9, 9, QMC::EnumIsScoped).add({
            {   16, Duration::Default },
            {   26, Duration::Short },
            {   27, Duration::Long },
        }),
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(11, 11, QMC::EnumIsScoped).add({
            {   16, Type::Default },
            {   28, Type::Video },
            {   29, Type::Playlist },
            {   30, Type::Channel },
            {   31, Type::All },
        }),
        // enum 'Feature'
        QtMocHelpers::EnumData<enum Feature>(32, 32, QMC::EnumIsScoped).add({
            {   33, Feature::Hd },
            {   34, Feature::Subtitles },
            {   35, Feature::CreativeCommons },
            {   36, Feature::ThreeD },
            {   37, Feature::Live },
            {   38, Feature::Purchased },
            {   39, Feature::FourK },
            {   40, Feature::ThreeSixty },
            {   41, Feature::Location },
            {   42, Feature::Hdr },
        }),
        // enum 'PipedFilter'
        QtMocHelpers::EnumData<enum PipedFilter>(15, 15, QMC::EnumIsScoped).add({
            {   31, PipedFilter::All },
            {   43, PipedFilter::Videos },
            {   44, PipedFilter::Channels },
            {   45, PipedFilter::Playlists },
            {   46, PipedFilter::MusicSongs },
            {   47, PipedFilter::MusicVideos },
            {   48, PipedFilter::MusicAlbums },
            {   49, PipedFilter::MusicPlaylists },
            {   50, PipedFilter::MusicArtists },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<SearchParameters, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject SearchParameters::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SearchParametersE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SearchParametersE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16SearchParametersE_t>.metaTypes,
    nullptr
} };

void SearchParameters::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SearchParameters *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->query(); break;
        case 1: *reinterpret_cast<enum SortBy*>(_v) = _t->sortBy(); break;
        case 2: *reinterpret_cast<enum Date*>(_v) = _t->date(); break;
        case 3: *reinterpret_cast<enum Duration*>(_v) = _t->duration(); break;
        case 4: *reinterpret_cast<enum Type*>(_v) = _t->type(); break;
        case 5: *reinterpret_cast<QList<Feature>*>(_v) = _t->features(); break;
        case 6: *reinterpret_cast<enum PipedFilter*>(_v) = _t->pipedFilter(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setQuery(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setSortBy(*reinterpret_cast<enum SortBy*>(_v)); break;
        case 2: _t->setDate(*reinterpret_cast<enum Date*>(_v)); break;
        case 3: _t->setDuration(*reinterpret_cast<enum Duration*>(_v)); break;
        case 4: _t->setType(*reinterpret_cast<enum Type*>(_v)); break;
        case 5: _t->setFeatures(*reinterpret_cast<QList<Feature>*>(_v)); break;
        case 6: _t->setPipedFilter(*reinterpret_cast<enum PipedFilter*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *SearchParameters::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SearchParameters::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SearchParametersE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SearchParameters::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
