/****************************************************************************
** Meta object code from reading C++ file 'config.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../config.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'config.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8SettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto Settings::qt_create_metaobjectdata<qt_meta_tag_ZN8SettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Settings",
        "QML.Element",
        "auto",
        "HideShortsChanged",
        "",
        "UseSponsorBlockChanged",
        "SkipSponsorBlockChanged",
        "PreferredVideoQualityChanged",
        "SponsorBlockServerChanged",
        "ProxyTypeChanged",
        "ProxyHostChanged",
        "ProxyPortChanged",
        "ProxyUserChanged",
        "ProxyPasswordChanged",
        "hideShorts",
        "isHideShortsImmutable",
        "defaultHideShortsValue",
        "useSponsorBlock",
        "isUseSponsorBlockImmutable",
        "defaultUseSponsorBlockValue",
        "skipSponsorBlock",
        "isSkipSponsorBlockImmutable",
        "defaultSkipSponsorBlockValue",
        "preferredVideoQuality",
        "isPreferredVideoQualityImmutable",
        "defaultPreferredVideoQualityValue",
        "sponsorBlockServer",
        "isSponsorBlockServerImmutable",
        "defaultSponsorBlockServerValue",
        "proxyType",
        "isProxyTypeImmutable",
        "defaultProxyTypeValue",
        "proxyHost",
        "isProxyHostImmutable",
        "defaultProxyHostValue",
        "proxyPort",
        "isProxyPortImmutable",
        "defaultProxyPortValue",
        "proxyUser",
        "isProxyUserImmutable",
        "proxyPassword",
        "isProxyPasswordImmutable",
        "EnumPreferredVideoQuality",
        "Q2160",
        "Q1440",
        "Q1080",
        "Q720",
        "Q480",
        "Q360",
        "Q240",
        "Q144",
        "EnumProxyType",
        "System",
        "HTTP"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'HideShortsChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'UseSponsorBlockChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SkipSponsorBlockChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'PreferredVideoQualityChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'SponsorBlockServerChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ProxyTypeChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ProxyHostChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ProxyPortChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ProxyUserChanged'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ProxyPasswordChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hideShorts'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isHideShortsImmutable'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultHideShortsValue'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'useSponsorBlock'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isUseSponsorBlockImmutable'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultUseSponsorBlockValue'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'skipSponsorBlock'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isSkipSponsorBlockImmutable'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultSkipSponsorBlockValue'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'preferredVideoQuality'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'isPreferredVideoQualityImmutable'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultPreferredVideoQualityValue'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'sponsorBlockServer'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isSponsorBlockServerImmutable'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultSponsorBlockServerValue'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'proxyType'
        QtMocHelpers::PropertyData<int>(29, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'isProxyTypeImmutable'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultProxyTypeValue'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'proxyHost'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'isProxyHostImmutable'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultProxyHostValue'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'proxyPort'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'isProxyPortImmutable'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultProxyPortValue'
        QtMocHelpers::PropertyData<int>(37, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'proxyUser'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'isProxyUserImmutable'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'proxyPassword'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'isProxyPasswordImmutable'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'EnumPreferredVideoQuality'
        QtMocHelpers::EnumData<enum EnumPreferredVideoQuality>(42, 42, QMC::EnumFlags{}).add({
            {   43, EnumPreferredVideoQuality::Q2160 },
            {   44, EnumPreferredVideoQuality::Q1440 },
            {   45, EnumPreferredVideoQuality::Q1080 },
            {   46, EnumPreferredVideoQuality::Q720 },
            {   47, EnumPreferredVideoQuality::Q480 },
            {   48, EnumPreferredVideoQuality::Q360 },
            {   49, EnumPreferredVideoQuality::Q240 },
            {   50, EnumPreferredVideoQuality::Q144 },
        }),
        // enum 'EnumProxyType'
        QtMocHelpers::EnumData<enum EnumProxyType>(51, 51, QMC::EnumFlags{}).add({
            {   52, EnumProxyType::System },
            {   53, EnumProxyType::HTTP },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Settings, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Settings::staticMetaObject = { {
    QMetaObject::SuperData::link<KConfigSkeleton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8SettingsE_t>.metaTypes,
    nullptr
} };

void Settings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Settings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->HideShortsChanged(); break;
        case 1: _t->UseSponsorBlockChanged(); break;
        case 2: _t->SkipSponsorBlockChanged(); break;
        case 3: _t->PreferredVideoQualityChanged(); break;
        case 4: _t->SponsorBlockServerChanged(); break;
        case 5: _t->ProxyTypeChanged(); break;
        case 6: _t->ProxyHostChanged(); break;
        case 7: _t->ProxyPortChanged(); break;
        case 8: _t->ProxyUserChanged(); break;
        case 9: _t->ProxyPasswordChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::HideShortsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::UseSponsorBlockChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::SkipSponsorBlockChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::PreferredVideoQualityChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::SponsorBlockServerChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::ProxyTypeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::ProxyHostChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::ProxyPortChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::ProxyUserChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Settings::*)()>(_a, &Settings::ProxyPasswordChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->hideShorts(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isHideShortsImmutable(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->defaultHideShortsValue(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->useSponsorBlock(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isUseSponsorBlockImmutable(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->defaultUseSponsorBlockValue(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->skipSponsorBlock(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isSkipSponsorBlockImmutable(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->defaultSkipSponsorBlockValue(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->preferredVideoQuality(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isPreferredVideoQualityImmutable(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->defaultPreferredVideoQualityValue(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->sponsorBlockServer(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isSponsorBlockServerImmutable(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->defaultSponsorBlockServerValue(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->proxyType(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isProxyTypeImmutable(); break;
        case 17: *reinterpret_cast<int*>(_v) = _t->defaultProxyTypeValue(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->proxyHost(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isProxyHostImmutable(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->defaultProxyHostValue(); break;
        case 21: *reinterpret_cast<int*>(_v) = _t->proxyPort(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->isProxyPortImmutable(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->defaultProxyPortValue(); break;
        case 24: *reinterpret_cast<QString*>(_v) = _t->proxyUser(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->isProxyUserImmutable(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->proxyPassword(); break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->isProxyPasswordImmutable(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHideShorts(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setUseSponsorBlock(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setSkipSponsorBlock(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setPreferredVideoQuality(*reinterpret_cast<int*>(_v)); break;
        case 12: _t->setSponsorBlockServer(*reinterpret_cast<QString*>(_v)); break;
        case 15: _t->setProxyType(*reinterpret_cast<int*>(_v)); break;
        case 18: _t->setProxyHost(*reinterpret_cast<QString*>(_v)); break;
        case 21: _t->setProxyPort(*reinterpret_cast<int*>(_v)); break;
        case 24: _t->setProxyUser(*reinterpret_cast<QString*>(_v)); break;
        case 26: _t->setProxyPassword(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Settings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Settings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SettingsE_t>.strings))
        return static_cast<void*>(this);
    return KConfigSkeleton::qt_metacast(_clname);
}

int Settings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KConfigSkeleton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void Settings::HideShortsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Settings::UseSponsorBlockChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Settings::SkipSponsorBlockChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Settings::PreferredVideoQualityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Settings::SponsorBlockServerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Settings::ProxyTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Settings::ProxyHostChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Settings::ProxyPortChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Settings::ProxyUserChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Settings::ProxyPasswordChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
