/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<preferences.h>)
#  include <preferences.h>
#endif
#if __has_include(<searchparameters.h>)
#  include <searchparameters.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_plasmatube_invidious()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<QInvidious::Preferences>("org.kde.plasmatube.invidious", 1);
    qmlRegisterTypesAndRevisions<SearchParameters>("org.kde.plasmatube.invidious", 1);
    qmlRegisterEnum<SearchParameters::SortBy>("SearchParameters::SortBy");
    qmlRegisterEnum<SearchParameters::Date>("SearchParameters::Date");
    qmlRegisterEnum<SearchParameters::Duration>("SearchParameters::Duration");
    qmlRegisterEnum<SearchParameters::Type>("SearchParameters::Type");
    qmlRegisterEnum<SearchParameters::Feature>("SearchParameters::Feature");
    qmlRegisterEnum<SearchParameters::PipedFilter>("SearchParameters::PipedFilter");
    QT_WARNING_POP
    qmlRegisterModule("org.kde.plasmatube.invidious", 1, 0);
}

static const QQmlModuleRegistration orgkdeplasmatubeinvidiousRegistration("org.kde.plasmatube.invidious", qml_register_types_org_kde_plasmatube_invidious);
