/****************************************************************************
** Meta object code from reading C++ file 'videosource.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasmatube-26.08.0/src/controllers/videosource.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'videosource.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11VideoSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto VideoSource::qt_create_metaobjectdata<qt_meta_tag_ZN11VideoSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "VideoSource",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Use from SourceManager",
        "urlChanged",
        "",
        "typeChanged",
        "credentialsChanged",
        "usernameChanged",
        "preferencesChanged",
        "finishedLoading",
        "subscriptionsChanged",
        "canLogInChanged",
        "importExportCompleted",
        "numSubscriptions",
        "importExportError",
        "reason",
        "prepareLogIn",
        "logOut",
        "isVideoWatched",
        "videoId",
        "markVideoWatched",
        "addToPlaylist",
        "plid",
        "supportsPopularPage",
        "supportsTrendingCategories",
        "importSubscriptions",
        "filePath",
        "exportSubscriptions",
        "SubscriptionListModel*",
        "model",
        "url",
        "type",
        "Type",
        "loggedIn",
        "canLogIn",
        "username",
        "preferences",
        "QInvidious::Preferences",
        "showRelatedVideos",
        "Invidious",
        "PeerTube",
        "Piped"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'urlChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'credentialsChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'usernameChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preferencesChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finishedLoading'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subscriptionsChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canLogInChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'importExportCompleted'
        QtMocHelpers::SignalData<void(int)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Signal 'importExportError'
        QtMocHelpers::SignalData<void(const QString &)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Method 'prepareLogIn'
        QtMocHelpers::MethodData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'logOut'
        QtMocHelpers::MethodData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'isVideoWatched'
        QtMocHelpers::MethodData<bool(const QString &)>(22, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 23 },
        }}),
        // Method 'markVideoWatched'
        QtMocHelpers::MethodData<void(const QString &)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Method 'addToPlaylist'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(25, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::QString, 23 },
        }}),
        // Method 'supportsPopularPage'
        QtMocHelpers::MethodData<bool() const>(27, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'supportsTrendingCategories'
        QtMocHelpers::MethodData<bool() const>(28, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'importSubscriptions'
        QtMocHelpers::MethodData<void(const QString &)>(29, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Method 'exportSubscriptions'
        QtMocHelpers::MethodData<void(SubscriptionListModel *, const QString &)>(31, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { QMetaType::QString, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'url'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'type'
        QtMocHelpers::PropertyData<enum Type>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'loggedIn'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'canLogIn'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'username'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'preferences'
        QtMocHelpers::PropertyData<QInvidious::Preferences>(40, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'showRelatedVideos'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(36, 36, QMC::EnumIsScoped).add({
            {   43, Type::Invidious },
            {   44, Type::PeerTube },
            {   45, Type::Piped },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<VideoSource, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject VideoSource::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11VideoSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11VideoSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11VideoSourceE_t>.metaTypes,
    nullptr
} };

void VideoSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VideoSource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->urlChanged(); break;
        case 1: _t->typeChanged(); break;
        case 2: _t->credentialsChanged(); break;
        case 3: _t->usernameChanged(); break;
        case 4: _t->preferencesChanged(); break;
        case 5: _t->finishedLoading(); break;
        case 6: _t->subscriptionsChanged(); break;
        case 7: _t->canLogInChanged(); break;
        case 8: _t->importExportCompleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->importExportError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->prepareLogIn(); break;
        case 11: _t->logOut(); break;
        case 12: { bool _r = _t->isVideoWatched((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->markVideoWatched((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->addToPlaylist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 15: { bool _r = _t->supportsPopularPage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->supportsTrendingCategories();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->importSubscriptions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->exportSubscriptions((*reinterpret_cast<std::add_pointer_t<SubscriptionListModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SubscriptionListModel* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::urlChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::typeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::credentialsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::usernameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::preferencesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::finishedLoading, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::subscriptionsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)()>(_a, &VideoSource::canLogInChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)(int )>(_a, &VideoSource::importExportCompleted, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoSource::*)(const QString & )>(_a, &VideoSource::importExportError, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QInvidious::Preferences >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->url(); break;
        case 1: *reinterpret_cast<enum Type*>(_v) = _t->type(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->loggedIn(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->canLogIn(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->username(); break;
        case 5: *reinterpret_cast<QInvidious::Preferences*>(_v) = _t->preferences(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->showRelatedVideos(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUrl(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setType(*reinterpret_cast<enum Type*>(_v)); break;
        case 5: _t->setPreferences(*reinterpret_cast<QInvidious::Preferences*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *VideoSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VideoSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11VideoSourceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int VideoSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void VideoSource::urlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void VideoSource::typeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void VideoSource::credentialsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void VideoSource::usernameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void VideoSource::preferencesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void VideoSource::finishedLoading()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void VideoSource::subscriptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void VideoSource::canLogInChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void VideoSource::importExportCompleted(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void VideoSource::importExportError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
