/****************************************************************************
** Meta object code from reading C++ file 'videocontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasmatube-26.08.0/src/controllers/videocontroller.h"
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'videocontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15VideoControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto VideoController::qt_create_metaobjectdata<qt_meta_tag_ZN15VideoControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "VideoController",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Access via PlasmaTube.VideoController",
        "videoModeChanged",
        "",
        "currentPlayerChanged",
        "currentVideoChanged",
        "openNormalPlayer",
        "openPiPPlayer",
        "positionChanged",
        "durationChanged",
        "playbackStateChanged",
        "skippedSponsorSegment",
        "enteredSponsorSegment",
        "position",
        "play",
        "videoId",
        "queueNext",
        "togglePlaying",
        "stop",
        "previous",
        "next",
        "videoMode",
        "VideoMode",
        "currentPlayer",
        "MpvObject*",
        "currentVideo",
        "VideoItem*",
        "videoModel",
        "VideoModel*",
        "videoQueue",
        "VideoQueue*",
        "Normal",
        "PictureInPicture"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'videoModeChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentPlayerChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentVideoChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openNormalPlayer'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openPiPPlayer'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'durationChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'playbackStateChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'skippedSponsorSegment'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enteredSponsorSegment'
        QtMocHelpers::SignalData<void(qreal)>(17, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 18 },
        }}),
        // Method 'play'
        QtMocHelpers::MethodData<void(const QString &)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Method 'queueNext'
        QtMocHelpers::MethodData<void(const QString &)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Method 'togglePlaying'
        QtMocHelpers::MethodData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'stop'
        QtMocHelpers::MethodData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'previous'
        QtMocHelpers::MethodData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'next'
        QtMocHelpers::MethodData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'videoMode'
        QtMocHelpers::PropertyData<enum VideoMode>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'currentPlayer'
        QtMocHelpers::PropertyData<MpvObject*>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'currentVideo'
        QtMocHelpers::PropertyData<VideoItem*>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'videoModel'
        QtMocHelpers::PropertyData<VideoModel*>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'videoQueue'
        QtMocHelpers::PropertyData<VideoQueue*>(34, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'VideoMode'
        QtMocHelpers::EnumData<enum VideoMode>(27, 27, QMC::EnumFlags{}).add({
            {   36, VideoMode::Normal },
            {   37, VideoMode::PictureInPicture },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<VideoController, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject VideoController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15VideoControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15VideoControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15VideoControllerE_t>.metaTypes,
    nullptr
} };

void VideoController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VideoController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->videoModeChanged(); break;
        case 1: _t->currentPlayerChanged(); break;
        case 2: _t->currentVideoChanged(); break;
        case 3: _t->openNormalPlayer(); break;
        case 4: _t->openPiPPlayer(); break;
        case 5: _t->positionChanged(); break;
        case 6: _t->durationChanged(); break;
        case 7: _t->playbackStateChanged(); break;
        case 8: _t->skippedSponsorSegment(); break;
        case 9: _t->enteredSponsorSegment((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 10: _t->play((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->queueNext((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->togglePlaying(); break;
        case 13: _t->stop(); break;
        case 14: _t->previous(); break;
        case 15: _t->next(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::videoModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::currentPlayerChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::currentVideoChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::openNormalPlayer, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::openPiPPlayer, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::positionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::durationChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::playbackStateChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)()>(_a, &VideoController::skippedSponsorSegment, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (VideoController::*)(qreal )>(_a, &VideoController::enteredSponsorSegment, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< MpvObject* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VideoItem* >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VideoModel* >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< VideoQueue* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum VideoMode*>(_v) = _t->videoMode(); break;
        case 1: *reinterpret_cast<MpvObject**>(_v) = _t->currentPlayer(); break;
        case 2: *reinterpret_cast<VideoItem**>(_v) = _t->currentVideo(); break;
        case 3: *reinterpret_cast<VideoModel**>(_v) = _t->videoModel(); break;
        case 4: *reinterpret_cast<VideoQueue**>(_v) = _t->videoQueue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVideoMode(*reinterpret_cast<enum VideoMode*>(_v)); break;
        case 1: _t->setCurrentPlayer(*reinterpret_cast<MpvObject**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *VideoController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VideoController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15VideoControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int VideoController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void VideoController::videoModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void VideoController::currentPlayerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void VideoController::currentVideoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void VideoController::openNormalPlayer()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void VideoController::openPiPPlayer()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void VideoController::positionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void VideoController::durationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void VideoController::playbackStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void VideoController::skippedSponsorSegment()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void VideoController::enteredSponsorSegment(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
