/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<channelcontroller.h>)
#  include <channelcontroller.h>
#endif
#if __has_include(<commentsmodel.h>)
#  include <commentsmodel.h>
#endif
#if __has_include(<config.h>)
#  include <config.h>
#endif
#if __has_include(<invidiousinstancesmodel.h>)
#  include <invidiousinstancesmodel.h>
#endif
#if __has_include(<logincontroller.h>)
#  include <logincontroller.h>
#endif
#if __has_include(<mpvobject.h>)
#  include <mpvobject.h>
#endif
#if __has_include(<peertubeinstancesmodel.h>)
#  include <peertubeinstancesmodel.h>
#endif
#if __has_include(<pipedinstancesmodel.h>)
#  include <pipedinstancesmodel.h>
#endif
#if __has_include(<plasmatube.h>)
#  include <plasmatube.h>
#endif
#if __has_include(<playlistsmodel.h>)
#  include <playlistsmodel.h>
#endif
#if __has_include(<searchmodel.h>)
#  include <searchmodel.h>
#endif
#if __has_include(<sourcemanager.h>)
#  include <sourcemanager.h>
#endif
#if __has_include(<subscriptioncontroller.h>)
#  include <subscriptioncontroller.h>
#endif
#if __has_include(<subscriptionlistmodel.h>)
#  include <subscriptionlistmodel.h>
#endif
#if __has_include(<videocontroller.h>)
#  include <videocontroller.h>
#endif
#if __has_include(<videolistmodel.h>)
#  include <videolistmodel.h>
#endif
#if __has_include(<videomodel.h>)
#  include <videomodel.h>
#endif
#if __has_include(<videoqueue.h>)
#  include <videoqueue.h>
#endif
#if __has_include(<videosource.h>)
#  include <videosource.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_plasmatube()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    QMetaType::fromType<AbstractInstancesModel *>().id();
    QMetaType::fromType<AbstractListModel *>().id();
    qmlRegisterTypesAndRevisions<ChannelController>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<CommentsModel>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<InvidiousInstancesModel>("org.kde.plasmatube", 1);
    QMetaType::fromType<KConfigSkeleton *>().id();
    QMetaType::fromType<KCoreConfigSkeleton *>().id();
    qmlRegisterEnum<KCoreConfigSkeleton::DisambiguateConstructor>("KCoreConfigSkeleton::DisambiguateConstructor");
    qmlRegisterTypesAndRevisions<LogInController>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<MpvObject>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<PeerTubeInstancesModel>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<PipedInstancesModel>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<PlasmaTube>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<PlaylistsModel>("org.kde.plasmatube", 1);
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    qmlRegisterTypesAndRevisions<SearchModel>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<Settings>("org.kde.plasmatube", 1);
    qmlRegisterEnum<Settings::EnumPreferredVideoQuality>("Settings::EnumPreferredVideoQuality");
    qmlRegisterEnum<Settings::EnumProxyType>("Settings::EnumProxyType");
    qmlRegisterTypesAndRevisions<SourceManager>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<SubscriptionController>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<SubscriptionListModel>("org.kde.plasmatube", 1);
    QMetaType::fromType<SubscriptionWatcher *>().id();
    qmlRegisterTypesAndRevisions<VideoController>("org.kde.plasmatube", 1);
    qmlRegisterEnum<VideoController::VideoMode>("VideoController::VideoMode");
    qmlRegisterTypesAndRevisions<VideoItem>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<VideoListModel>("org.kde.plasmatube", 1);
    qmlRegisterEnum<VideoListModel::QueryType>("VideoListModel::QueryType");
    qmlRegisterTypesAndRevisions<VideoModel>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<VideoQueue>("org.kde.plasmatube", 1);
    qmlRegisterTypesAndRevisions<VideoSource>("org.kde.plasmatube", 1);
    qmlRegisterEnum<VideoSource::Type>("VideoSource::Type");
    QT_WARNING_POP
    qmlRegisterModule("org.kde.plasmatube", 1, 0);
}

static const QQmlModuleRegistration orgkdeplasmatubeRegistration("org.kde.plasmatube", qml_register_types_org_kde_plasmatube);
